@@ -1,8 +1,8 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
-from datetime import datetime, timedelta
-
+from datetime import timedelta
+import pandas as pd
 from backtrader.feed import DataBase
 from backtrader import date2num, num2date
 from backtrader.utils.py3 import queue, with_metaclass
@@ -133,6 +133,7 @@ class AlpacaData(with_metaclass(MetaAlpacaData, DataBase)):
         ('reconnect', True),
         ('reconnections', -1),  # forever
         ('reconntimeout', 5.0),
+        ('data_feed', 'iex'),  # options iex/sip for pro
     )
 
     _store = alpacastore.AlpacaStore
@@ -233,7 +234,8 @@ class AlpacaData(with_metaclass(MetaAlpacaData, DataBase)):
             return True
         self.qlive = self.o.streaming_prices(self.p.dataname,
                                              self.p.timeframe,
-                                             tmout=tmout)
+                                             tmout=tmout,
+                                             data_feed=self.p.data_feed)
         if instart:
             self._statelivereconn = self.p.backfill_start
         else:
@@ -338,7 +340,7 @@ class AlpacaData(with_metaclass(MetaAlpacaData, DataBase)):
                     # passing None to fetch max possible in 1 request
                     dtbegin = None
 
-                dtend = datetime.utcfromtimestamp(int(msg['time']))
+                dtend = pd.Timestamp(msg['time'], unit='ns')
 
                 self.qhist = self.o.candles(
                     self.p.dataname, dtbegin, dtend,
@@ -401,7 +403,7 @@ class AlpacaData(with_metaclass(MetaAlpacaData, DataBase)):
                     return False
 
     def _load_tick(self, msg):
-        dtobj = datetime.utcfromtimestamp(msg['time'])
+        dtobj = pd.Timestamp(msg['time'], unit='ns')
         dt = date2num(dtobj)
         if dt <= self.lines.datetime[-1]:
             return False  # time already seen
@@ -413,8 +415,8 @@ class AlpacaData(with_metaclass(MetaAlpacaData, DataBase)):
 
         # Put the prices into the bar
         tick = float(
-            msg['askprice']) if self.p.useask else float(
-            msg['bidprice'])
+            msg['ask_price']) if self.p.useask else float(
+            msg['bid_price'])
         self.lines.open[0] = tick
         self.lines.high[0] = tick
         self.lines.low[0] = tick
@@ -425,7 +427,7 @@ class AlpacaData(with_metaclass(MetaAlpacaData, DataBase)):
         return True
 
     def _load_agg(self, msg):
-        dtobj = datetime.utcfromtimestamp(int(msg['time']))
+        dtobj = pd.Timestamp(msg['time'], unit='ns')
         dt = date2num(dtobj)
         if dt <= self.lines.datetime[-1]:
             return False  # time already seen
